#!/bin/sh
#
# RECIPIENTS (UN)INSTALL Script (recipients.sh)
# -----------------------------------------------
#
# Purpose:      To install and uninstall the RECIPIENTS patch
#
# Parameters:   -u (uninstall)
#	        VRF (Version to be uninstalled)
#
# Usage:        ./install_recipients.sh [-u] [Version]
#
#		Installation: 	./recipients.sh
# 		Uninstallation: ./recipients.sh -u 105
#
# Return Codes: 0 - Patches applied successfully
#		1 - Original QMAIL files not found (Patch not extracted in QMAIL source directory)
#		2 - Patch files not found 
#
# Output:	recipients.log
#
# History:      1.0.1 - Erwin Hoffmann - Initial release
#
#---------------------------------------------------------------------------------------
#
DATE=$(date)
LOCDIR=${PWD}
QMAILHOME=$(head -1 conf-qmail)
SOLARIS=$(sh ./find-systype.sh | grep -ciq "SunOS")
LOGFILE=recipients.log
TARGETS=FILES.recipients
IFSKEEP=${IFS}
REL=015 # Should be identical to RECIPIENTS level
BUILD=2003357161429


if [ $# -eq 0 ] ; then

	echo "Installing RECIPIENTS $REL (Build $BUILD) at $DATE <<<" | tee -a $LOGFILE 2>&1 
	
	for FILE in $(grep "^= " ${TARGETS} | awk '{print $2}'); do
		echo "Targeting file $FILE ..." | tee -a $LOGFILE 2>&1
		if [ -s ${FILE} ] ; then
			cp ${FILE} ${FILE}.$REL | tee -a $LOGFILE 2>&1
			echo "--> ${FILE} copied to ${FILE}.$REL" | tee -a $LOGFILE 2>&1
		else
			echo "${FILE} not found!"
			exit 1
		fi
		if [ -s ${FILE}.patch ] ; then
			if [ ${SOLARIS} -gt 0 ]; then
				echo "--> Patching qmail source file ${FILE} for Solaris ...." | tee -a $LOGFILE 2>&1
				patch -i ${FILE}.patch ${FILE} 2>&1 | tee -a $LOGFILE
			else
				echo "--> Patching qmail source file ${FILE}  ...." | tee -a $LOGFILE 2>&1
				patch ${FILE} ${FILE}.patch 2>&1 | tee -a $LOGFILE
			fi
		else
			echo "!! ${FILE}.patch not found / not applicable !"
			exit 2
		fi
	done 

	echo "Copying documentation and samples to ${QMAILHOME}/doc/ ..." | tee -a $LOGFILE 2>&1 

	cp -v [A-Z]*.recipients ${QMAILHOME}/doc/ | tee -a $LOGFILE 2>&1
        cp -v qmail-*2recipients ${QMAILHOME}/doc | tee -a $LOGFILE 2>&1
	echo "Installation of RECIPIENTS $REL (Build $BUILD) finished at $DATE <<<" | tee -a $LOGFILE 2>&1 

# Now go for the uninstallation....

elif [ "$1" = "-u" ] ; then

# Get the Version Number from INPUT 

	if [ $# -eq 2 ] ; then
		REL=$2
	fi

	echo "De-installing RECIPIENTS $REL (Build $BUILD) at $DATE <<<" | tee -a $LOGFILE 2>&1 

	for FILE in $(grep "^= " ${TARGETS} | awk '{print $2}'); do
		echo "Targeting file $FILE ..." | tee -a $LOGFILE 2>&1
		if [ -s ${FILE}.$REL ] ; then
			mv ${FILE}.$REL ${FILE} | tee -a $LOGFILE 2>&1
			touch ${FILE}
			echo "--> ${FILE}.$REL moved to ${FILE}" | tee -a $LOGFILE 2>&1
		else
			echo "!! ${FILE}.$REL not found !"
		fi
	done
	echo "De-installation of RECIPIENTS $REL (Build $BUILD) finished at $DATE <<<" | tee -a $LOGFILE 2>&1 
fi

exit 0
