#include "cdb.h"
#include "byte.h"
#include "open.h"
#include "error.h"
#include "control.h"
#include "constmap.h"
#include "stralloc.h"
#include "recipients.h"
#include "strerr.h"
#include "exit.h"

#define VERP
#define AUTO_BREAK '-'
#define LOCALS

#ifdef LOCALS
static int flaglocals = 0;
static stralloc locals = {0};
static struct constmap maplocals;
#endif
static stralloc key = {0};
static stralloc domain = {0};
static stralloc recipient = {0};
static stralloc rcpts = {0};
static struct constmap maprcpts;
static int flagrcpts = 0;
static int fdrcps;

/* return -2: out of memory */
/* return -1: no control file */
/* return  0: address not found; fatal */
/* return  1: address in recipients */
/* return  2: address not to be checked */

int recipients_init()
{
#ifdef LOCALS
  flaglocals = control_readfile(&locals,"control/locals",0);
  if (flaglocals != 1) return flaglocals;
  if (!constmap_init(&maplocals,locals.s,locals.len,0)) return flaglocals = -1;
#endif
  flagrcpts = control_readfile(&rcpts,"control/recipients",0);
  if (flagrcpts !=1) return flagrcpts;
}

int recipients(buf,len)
char *buf;
int len;
{
  int at;
  int ab;
  int i;

  if (flagrcpts != 1) return 2;

#ifdef LOCALS
  if (flaglocals != 1) return 1;
#endif
  byte_zero(recipient.s,recipient.len);		/* multiple recipients */
  byte_zero(domain.s,domain.len);

  at = byte_rchr(buf,len,'@');
  if (at < len) {
#ifdef LOCALS
    if (!constmap(&maplocals,buf + at + 1,len - at - 1)) return 2; 
#endif
    if (!stralloc_copyb(&domain,buf + at,len - at)) return -2;
    if (!stralloc_copyb(&recipient,buf,len)) return -2;
  } else {
    if (!stralloc_copyb(&recipient,buf,len)) return -2;
    if (!stralloc_copys(&domain,"@localhost")) return -2;
    if (!stralloc_cat(&recipient,&domain)) return -2; 
  }

  byte_zero(key.s,key.len);	
  if (!stralloc_copys(&key,":")) return -2;
  if (!stralloc_cat(&key,&recipient)) return -2;
  if (!stralloc_0(&key)) return -2;
  case_lowerb(key.s,key.len);

  if (recipients_parse(rcpts.s,key.s) == 1) return 1;

#ifdef VERP
  for (i = 0; i < at; i++) {
    ab = 0;
    if (buf[i] == AUTO_BREAK) 
      ab = i;
    if (ab) {
      byte_zero(key.s,key.len);			
      if (!stralloc_copyb(&recipient,buf,ab)) return -2;
      if (!stralloc_cat(&recipient,&domain)) return -2;
      if (!stralloc_copys(&key,":")) return -2;
      if (!stralloc_cat(&key,&recipient)) return -2;
      if (!stralloc_0(&key)) return -2;
      case_lowerb(key.s,key.len);

      if (recipients_parse(rcpts.s,key.s) == 1) return 1;
    }
  }
#endif

  return 0;
} 
int recipients_parse (rcps,rkey)
char *rcps;
char *rkey;
{
  for (;;) {         
    fdrcps = open_read(rcps);
    if (fdrcps != -1) {
      uint32 dlen;
      int r;
      r = cdb_seek(fdrcps,rkey,str_len(rkey),&dlen);
      close(fdrcps);
      if (r) return 1;
    }
    rcps = rcps + str_len(rcps) + 1;
    if (*rcps == 'Z' || *rcps == ' ') return 0;
  }
  return 0;
}
